# -*- coding: UTF-8 -*-
# ServerSystem属于SDK的内部类,用于绑定系统
from mod.server.system.serverSystem import ServerSystem
from mod.server.serverEvent import ServerEvent
from mod.common.minecraftEnum import ItemPosType
import mod.server.extraServerApi as ServerApi


# 继承ServerSystem这个类，并在魔法方法__init__内调用ServerSystem的__init__方法，来初始化这个Main类
class Main(ServerSystem):

    def __init__(self, namespace, system_name):
        ServerSystem.__init__(self, namespace, system_name)
        # 获取脚本引擎自己的命名空间
        namespace = ServerApi.GetEngineNamespace()
        # 获取脚本引擎自己的系统名称
        system_name = ServerApi.GetEngineSystemName()
        # 监听ServerChatEvent聊天事件，从文档里可得知
        self.ListenForEvent(namespace, system_name, ServerEvent.ServerChatEvent, self, self.player_chat)
        # 再次监听ServerChatEvent聊天事件，这里分开功能，去执行传送点的操作
        self.ListenForEvent(namespace, system_name, ServerEvent.ServerChatEvent, self, self.player_teleport)
        # 监听PlayerAttackEntityEvent玩家攻击实体事件，从文档里可得知
        self.ListenForEvent(namespace, system_name, ServerEvent.PlayerAttackEntityEvent, self, self.player_attacked)
        print '[SDK]>>>>>>Tutorial step two server init'
        # 用来暂时缓存玩家的传送点信息，在每次重新开启一次游戏后会被重置
        self.player_tp_cache = {}

    def player_chat(self, event):
        # 获取玩家ID
        player_id = event['playerId']
        # 获取聊天内容
        message = event['message']
        # 设置前缀
        result = ServerApi.GetEngineCompFactory().CreateName(player_id).SetPlayerPrefixAndSuffixName(message, '', '', '')
        # 打印出前缀设置结果
        print '[SDK]>>>>>>set prefix ', result
        # 如果聊天信息为以下内容
        if message == '第一人称':
            self.NotifyToClient(player_id, 'ChangePerspective', {'perspective': 0})
        elif message == '第三人称':
            self.NotifyToClient(player_id, 'ChangePerspective', {'perspective': 1})
        elif message == '前视第三人称':
            self.NotifyToClient(player_id, 'ChangePerspective', {'perspective': 2})

    def player_teleport(self, event):
        # 获取玩家ID
        player_id = event['playerId']
        # 获取聊天内容
        message = event['message']
        if message.startswith('传送点设置'):
            pos_comp = ServerApi.GetEngineCompFactory().CreatePos(player_id)
            teleport_pos = pos_comp.GetFootPos()
            # 保存玩家的当前坐标至字典缓存中
            self.player_tp_cache[player_id] = teleport_pos
            print 'player teleport config'
        if message.startswith('传送点传送'):
            pos_comp = ServerApi.GetEngineCompFactory().CreatePos(player_id)
            # 尝试提取玩家缓存的传送点信息
            if player_id in self.player_tp_cache:
                teleport_pos = self.player_tp_cache[player_id]
                # 传送玩家
                pos_comp.SetPos(teleport_pos)
                print 'player teleport finish'

    def player_attacked(self, event):
        # 获取玩家ID
        player_id = event['playerId']
        # 获取受害者ID
        victim_id = event['victimId']
        # 获取玩家手持物品
        item_comp = ServerApi.GetEngineCompFactory().CreateItem(player_id)
        carried_item = item_comp.GetPlayerItem(ItemPosType.CARRIED, 0)
        # 判断物品类型
        if carried_item and carried_item['itemName'] == 'minecraft:stick':
            # 获得玩家的头部转向
            rot_comp = ServerApi.GetEngineCompFactory().CreateRot(player_id)
            victim_rot = rot_comp.GetRot()
            # 将头部转向变为方向向量
            victim_dir = ServerApi.GetDirFromRot(victim_rot)
            # 为受害者创建击退接口，在参数内放入向量，并给予一个强度
            actor_comp = ServerApi.GetEngineCompFactory().CreateAction(victim_id)
            actor_comp.SetMobKnockback(victim_dir[0], victim_dir[2], 5.0)
